# file: life.py
# author: Bob Muller
#
# CS1101 Computer Science I
#
# Code for Conway's Game of Life. The initial generation is provided
# by an external text file. A * denotes alive, anothing else, isn't.
#
from stddraw import *

displayWidth = 600
displayHeight = displayWidth

# type colony = (bool list) list    (i.e., a 2D array of booleans)

# readColony : string -> colony
#
def readColony(filename):
    def fillRow(string):
        row = []
        for letter in string:
            row.append(letter == "*")
        return [False] + row + [False]    
    infile = open(filename, "r")
    lines = [nextline.strip() for nextline in infile]
    infile.close()
    
    colony = []
    for row in range(len(lines)):
        colony.append(fillRow(lines[row]))
        
    outerWall = [False] * (len(lines) + 2)
    return [outerWall] + colony + [outerWall]


# countNeighbors : colony * int * int -> int
#
def countNeighbors(colony, row, col):
    count = 0
    for r in range(row - 1, row + 2):
        for c in range(col - 1, col + 2):
            if colony[r][c]:
                count = count + 1
    if colony[row][col]:
        count = count - 1
    return count

# make2DArray : int * int * A -> (A list) list
#
def make2DArray(rows, cols, value):
    return [ [ value for _ in range(cols) ] for _ in range(rows) ]

# nextGeneration : colony * colony -> colony * colony
#
# Given a call nextGeneration(colony, workArea), this function toggles
# back and forth between colony and workArea.
#
def nextGeneration(colony, workArea):
    for row in range(1, len(colony) - 1):
        for col in range(1, len(colony[row]) - 1):
            neighbors = countNeighbors(colony, row, col)
            if colony[row][col]:
                workArea[row][col] = (neighbors == 2) or (neighbors == 3)
            else:
                workArea[row][col] = neighbors == 3
    return (workArea, colony)

# getColor : colony * int * int -> string
#
def getColor(board, row, col): return "red" if board[row][col] else "grey"

# render : Picture * colony -> void
#
def render(picture, colony):
    side = 1.0 / len(colony)
    halfSide = side / 2.0
    for row in range(len(colony)):
        for col in range(len(colony[0])):
            color = getColor(colony, row, col)
            (x, y) = (col * side + halfSide, row * side + halfSide)
            picture.filledSquare(x, y, halfSide, color)

# evolve : Picture * colony * colony -> void
#
def evolve(picture, board, workArea):
    while True:
        render(picture, board)
        picture.wait(1)
        (board, workArea) = nextGeneration(board, workArea)
        picture.clear()

# go : string -> void
#
def go(filename):
    picture = Picture()
    def responder(event):
        colony = readColony(filename)
        workArea = make2DArray(len(colony), len(colony[0]), False)
        evolve(picture, colony, workArea)

    picture.setW(displayWidth)
    picture.setH(displayHeight)
    picture.bind('<Button-1>', responder)
    picture.start()

go("two.txt")
